/*--------------------------------------------------
   Basisroutinen zur Datenbertragung <FT_BASE.C>
--------------------------------------------------*/

/*-------------------
   Include-Dateien
-------------------*/
#include <dos.h>
#include <stdio.h>
#include <ET-SER.DEF>

#define	uchr		unsigned char
#define	uint		unsigned int

/*--------------
   Prototypen
--------------*/
void			SetAlarm(int sek),
			ComFlush(void);
int			Port = COM_2,
			Baud = B_1200,
			Data = DATA_8,
			Pari = PAR_NO,
			Stop = STOP_1;
int			Is_Alarm(void),
			ComRead(uchr *buf, int len),
			ComWrite(uchr *buf, int len),
			ComOpen(int comx, int baud, int data, int pari, int stop);
static uint		getfmt(void);
static void 	setfmt(int format);
static int 		comget(void);

/*---------------------
   Globale Variablen
---------------------*/
static uint		Baud_F[] = {0x0180, 0x00C0,
					0x0060, 0x0030,
					0x0018, 0x000C,
					0x0006, 0x0003,
					0x0002, 0x0001},
			ComBase=0,	/* Kommunikationsportverweis */
			TimeOut=2;
static long		Startsek,
			Alarmsek;

/*--------------------------------
   Initialisiert den Alarmtimer
--------------------------------*/
void	SetAlarm(int sek)
{	if (sek > 0)
	{	Startsek = (long)peek(0,TIMER_HI) * 65536L + peek(0,TIMER_LO);
		Alarmsek = (long)sek * 18;
	}
	else Startsek = Alarmsek = 0L;
}

int	Is_Alarm()
{	if (Alarmsek)
	{	if ((Startsek+Alarmsek) > (long)peek(0,TIMER_HI)*65536L + peek(0,TIMER_LO))
			return(0);
		SetAlarm(0);
	}
	return(1);
}

static uint getfmt()
{	return((uint)inport(ComBase + DATA_F));
}

static void setfmt(int format)
{	outport(ComBase + DATA_F,format);
}

int	ComOpen(int comx, int baud, int data, int pari, int stop)
{     uint		parm=0;

	if (comx < COM_1 || comx > COM_4) return(PORT_ERR);
	else ComBase = peek(0x0040,comx*2);

	if (baud < B_300 || baud > B_115200) return(BAUD_ERR);
	else
	{	uint newbaud = Baud_F[baud];
		setfmt(getfmt() | 0x80);
		outport(ComBase + BAUD_LSB,newbaud & 0x00FF);
		outport(ComBase + BAUD_MSB,newbaud >> 8);
		setfmt(getfmt() & 0x7F);
	}

	if (data != DATA_7 && data != DATA_8) return(DATA_ERR);
	else parm = data - 5;

	if (pari < PAR_NO || pari > PAR_EVEN) return(PARI_ERR);
	else
	{	if (pari == 2) pari = 3;
		parm |= (pari << 3);
	}

	if (stop != STOP_1 && stop != STOP_2) return(STOP_ERR);
	else
	{	stop -= 1;
		parm |= (stop << 2);
	}

	setfmt(parm);
	return(0);
}

static int comget()
{	return((inport(ComBase + SIO_STAT) & RCV_MASK) ? (inport(ComBase)) : -1);
}

void	ComFlush()
{	int		loop=0;

	while (loop++ < RETRY)
		if (comget() != -1) loop = 0;
}

int	ComRead(uchr *buf, int len)
{	int	ret=0,
		try=0;

	while (ret < len)
	{	if (inport(ComBase+SIO_STAT) & RCV_MASK)
		{	*buf++ = inport(ComBase);
			if (try > RETRY) SetAlarm(0);
			try = 0;
			ret++;
		}
		else
		{	if (try > RETRY)
			{	if (Is_Alarm()) break;
			}
			else if (++try > RETRY) SetAlarm(TimeOut);
		}
	}
	return(ret);
}

int	ComWrite(uchr *buf, int len)
{	int	ret=0,
		try=0;

	while (ret < len)
	{	if (inport(ComBase+SIO_STAT) & SND_MASK)
		{	outport(ComBase,(int) *buf++);
			if (try > RETRY) SetAlarm(0);
			try = 0;
			ret++;
		}
		else
		{	if (try > RETRY)
			{	if (Is_Alarm()) break;
			}
			else if (++try > RETRY) SetAlarm(TimeOut);
		}
	}
	return(ret);
}

main (void)
{
	ComOpen(Port, Baud, Data, Pari, Stop);

	return (0);
}
